#[cfg(web_sys_unstable_apis)]
#[doc = ""]
#[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
#[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
pub mod gpu_buffer_usage {
    #![allow(unused_imports)]
    #![allow(clippy::all)]
    use super::super::*;
    use wasm_bindgen::prelude::*;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.MAP_READ` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const MAP_READ: u32 = 1u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.MAP_WRITE` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const MAP_WRITE: u32 = 2u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.COPY_SRC` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const COPY_SRC: u32 = 4u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.COPY_DST` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const COPY_DST: u32 = 8u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.INDEX` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const INDEX: u32 = 16u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.VERTEX` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const VERTEX: u32 = 32u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.UNIFORM` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const UNIFORM: u32 = 64u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.STORAGE` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const STORAGE: u32 = 128u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.INDIRECT` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const INDIRECT: u32 = 256u64 as u32;
    #[cfg(web_sys_unstable_apis)]
    #[doc = "The `GPUBufferUsage.QUERY_RESOLVE` const."]
    #[doc = ""]
    #[doc = "*This API requires the following crate features to be activated: `gpu_buffer_usage`*"]
    #[doc = ""]
    #[doc = "*This API is unstable and requires `--cfg=web_sys_unstable_apis` to be activated, as"]
    #[doc = "[described in the `wasm-bindgen` guide](https://rustwasm.github.io/docs/wasm-bindgen/web-sys/unstable-apis.html)*"]
    pub const QUERY_RESOLVE: u32 = 512u64 as u32;
}
